/*================================================================================
	
	---------------------------
	-*- [ZP] Grenade: smoke -*-
	---------------------------
	
	This plugin is part of Zombie Plague Mod and is distributed under the
	terms of the GNU General Public License. Check ZP_ReadMe.txt for details.
	
================================================================================*/

#include <amxmodx>
#include <fakemeta>
#include <hamsandwich>

#include <amx_settings_api>
#include <cs_ham_bots_api>
#include <cs_weap_models_api>
#include <zombieplague>

#include <bitsums>
//#include <d7_debug>

// Settings file
new const ZP_SETTINGS_FILE[] = "zp_grenade_smoke.ini"

#define MODEL_MAX_LENGTH 64
#define SOUND_MAX_LENGTH 64
#define SPRITE_MAX_LENGTH 64

// Models
new g_model_grenade_smoke[MODEL_MAX_LENGTH] = "models/v_hegrenade.mdl"

// Sprites
new g_sprite_grenade_trail[SPRITE_MAX_LENGTH] = "sprites/laserbeam.spr"
new g_sprite_grenade_explode[SPRITE_MAX_LENGTH] = "sprites/zerogxplode.spr"

// HACK: pev_ field used to store custom nade types and their values
const PEV_NADE_TYPE = pev_flTimeStepSound
const NADE_TYPE_smoke = 4444 + 1;

new g_trailSpr, g_iIDSpriteExplode

new cvar_grenade_smoke_duration, cvar_grenade_smoke_radius

new g_iBsZombie, g_iIDInflictor[33];

public plugin_init()
{
	register_plugin("[ZP] Grenade: Smoke", "0.1.6", "ZP Dev Team, D i 5 7 i n c T")
	
	register_forward(FM_SetModel, "fwFmSetModelPre")
	
	RegisterHam(Ham_Think, "grenade", "fwHamThinkGrenadePre")
	RegisterHam(Ham_Spawn, "player", "fwHamSpawnPlayer", 1)
	RegisterHamBots(Ham_Spawn, "fwHamSpawnPlayer", 1)
	RegisterHam(Ham_TakeDamage, "player", "fwHamTakeDamagePlayer", 1)
	RegisterHamBots(Ham_TakeDamage, "fwHamTakeDamagePlayer", 1)
	
	cvar_grenade_smoke_duration = register_cvar("zp_grenade_smoke_damage", "600")
	cvar_grenade_smoke_radius = register_cvar("zp_grenade_smoke_radius", "300.0")
}

public plugin_precache()
{
	// Load from external file, save if not found
	if (!amx_load_setting_string(ZP_SETTINGS_FILE, "Models", "VIEW", g_model_grenade_smoke, charsmax(g_model_grenade_smoke)))
		amx_save_setting_string(ZP_SETTINGS_FILE, "Models", "VIEW", g_model_grenade_smoke)
	if (!amx_load_setting_string(ZP_SETTINGS_FILE, "Sprites", "TRAIL", g_sprite_grenade_trail, charsmax(g_sprite_grenade_trail)))
		amx_save_setting_string(ZP_SETTINGS_FILE, "Sprites", "TRAIL", g_sprite_grenade_trail)
	if (!amx_load_setting_string(ZP_SETTINGS_FILE, "Sprites", "EXPLODE", g_sprite_grenade_explode, charsmax(g_sprite_grenade_explode)))
		amx_save_setting_string(ZP_SETTINGS_FILE, "Sprites", "EXPLODE", g_sprite_grenade_explode)
	
	// Precache models
	precache_model(g_model_grenade_smoke)
	g_trailSpr = precache_model(g_sprite_grenade_trail)
	g_iIDSpriteExplode = precache_model(g_sprite_grenade_explode)
}

public fwHamSpawnPlayer(const iID)
{
	if (!is_user_alive(iID))
		return;
	
	if (zp_get_user_zombie(iID))
		return;
	
	bitsum_del(g_iBsZombie, iID)
	
	// Set custom grenade model
	cs_set_player_view_model(iID, CSW_SMOKEGRENADE, g_model_grenade_smoke)
}

public fwHamTakeDamagePlayer(const iIDVictim, const iIDInflictor, const iIDAttacker, const Float:fDamage, const iBsDamageType)
{
	if (iIDInflictor <= 0 || iIDInflictor != g_iIDInflictor[iIDVictim])
		return;
	
	//pev(iIDVictim, pev_dmg_take, fDamage)
	
	if (GetHamReturnStatus() == HAM_SUPERCEDE)//fDamage <= 0
		return;
	
	//new Float:fHealth;
	//pev(iIDVictim, pev_health, fHealth)
	//
	//ftD7Log(_, _, "[fwHamTakeDamagePlayer] fHealth: %f. fDamage: %f. fHealth - fDamage: %f. pev(iIDVictim, pev_health): %f(%d). pev(iIDVictim, pev_health) - fDamage: %f(%d). > 0: %d. > 0.0: %d.",
	//fHealth, fDamage, fHealth - fDamage, pev(iIDVictim, pev_health), pev(iIDVictim, pev_health), pev(iIDVictim, pev_health) - fDamage, pev(iIDVictim, pev_health) - fDamage,
	//(pev(iIDVictim, pev_health) - fDamage > 0), (pev(iIDVictim, pev_health) - fDamage > 0.0))
	//
	if (pev(iIDVictim, pev_health) > 0)
	{
		static iIDMsgScreenFade;
		if (!iIDMsgScreenFade)
			iIDMsgScreenFade = get_user_msgid("ScreenFade");
		
		message_begin(MSG_ONE_UNRELIABLE, iIDMsgScreenFade, _, iIDVictim)
		write_short(2048)
		write_short(1024)
		write_short(0x0000)
		write_byte(220)
		write_byte(0)
		write_byte(0)
		write_byte(205)
		message_end();
		
		//if (!is_user_connected(iIDAttacker))
			//return;
		
		//static szName[32];
		//get_user_name(iIDVictim, szName, charsmax(szName))
		
		//ftClientPrintChatColor(iIDAttacker, iIDVictim, "Damage done to ^"^3%s^1^" :: ^4%d^1.", szName, floatround(fDamage))
	}
	else
	{
		static iIDMessageDeath;
		if (!iIDMessageDeath)
			iIDMessageDeath = get_user_msgid("DeathMsg");
		
		message_begin(MSG_ALL, iIDMessageDeath)
		write_byte(iIDAttacker)
		write_byte(iIDVictim)
		write_byte(0) //headshot
		write_string("grenade")
		message_end()
	}
}

public zp_user_humanized_post(iID)
{
	bitsum_del(g_iBsZombie, iID)
	
	// Set custom grenade model
	cs_set_player_view_model(iID, CSW_SMOKEGRENADE, g_model_grenade_smoke)
}

public zp_user_infected_pre(iID)
{
	// Remove custom grenade model
	cs_reset_player_view_model(iID, CSW_SMOKEGRENADE)
}

public zp_user_infected_post(iID)
{
	bitsum_add(g_iBsZombie, iID)
}

// Forward Set Model
public fwFmSetModelPre(entity, const model[])
{
	// We don't care
	if (strlen(model) < 8)
		return;
	
	// Narrow down our matches a bit
	if (model[7] != 'w' || model[8] != '_')
		return;
	
	// Get damage time of grenade
	static Float:dmgtime
	pev(entity, pev_dmgtime, dmgtime)
	
	// Grenade not yet thrown
	if (dmgtime == 0.0)
		return;
	
	// Grenade's owner is zombie?
	if (zp_get_user_zombie(pev(entity, pev_owner)))
		return;
	
	// Smoke Grenade
	if (model[9] == 's' && model[10] == 'm')
	{
		set_pev(entity, pev_dmgtime, get_gametime() + 1.5)
		
		// Give it a glow
		fm_set_rendering(entity, kRenderFxGlowShell, 255, 0, 0, kRenderNormal, 16);
		
		// And a colored trail
		message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
		write_byte(TE_BEAMFOLLOW) // TE iID
		write_short(entity) // entity
		write_short(g_trailSpr) // sprite
		write_byte(10) // life
		write_byte(10) // width
		write_byte(255) // r
		write_byte(0) // g
		write_byte(0) // b
		write_byte(200) // brightness
		message_end()
		
		// Set grenade type on the thrown grenade entity
		set_pev(entity, PEV_NADE_TYPE, NADE_TYPE_smoke)
	}
/*	else if (model[9] == 'h' && model[10] == 'e')
	{
		ftClientPrintChatColor(0, _, "He grenade damage time: %f.", dmgtime - get_gametime())
	}*/
}

// Ham Grenade Think Forward
public fwHamThinkGrenadePre(entity)
{
	// Invalid entity
	if (!pev_valid(entity)) return HAM_IGNORED;
	
	// Get damage time of grenade
	static Float:dmgtime
	pev(entity, pev_dmgtime, dmgtime)
	
	new Float:current_time = get_gametime()
	
	// Check if it's time to go off
	if (dmgtime > current_time)
		return HAM_IGNORED;
	
	// Check if it's one of our custom nades
	switch (pev(entity, PEV_NADE_TYPE))
	{
		case NADE_TYPE_smoke: // smoke
		{
			static Float:fVecOrigin[3], Float:fVecOrigin2[3];
			pev(entity, pev_origin, fVecOrigin)
			
			message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
			write_byte(TE_EXPLOSION)
			engfunc(EngFunc_WriteCoord, fVecOrigin[0])
			engfunc(EngFunc_WriteCoord, fVecOrigin[1])
			engfunc(EngFunc_WriteCoord, fVecOrigin[2])
			write_short(g_iIDSpriteExplode)
			write_byte(30)
			write_byte(15)
			write_byte(0)
			message_end()
			
			static iMaxPlayers;
			if (!iMaxPlayers)
				iMaxPlayers = get_maxplayers();
			
			new iIDEntOwner = pev(entity, pev_owner),
			Float:fDistance,
			Float:fCvarDamage = get_pcvar_float(cvar_grenade_smoke_duration),
			Float:fCvarRadius = get_pcvar_float(cvar_grenade_smoke_radius);
			
			for (new iID = 1; iID <= iMaxPlayers; iID++)
			{
				if (iID == iIDEntOwner || !is_user_alive(iID) || !bitsum_get(g_iBsZombie, iID))
					continue;
				
				pev(iID, pev_origin, fVecOrigin2)
				fDistance = get_distance_f(fVecOrigin, fVecOrigin2);
				
				if (fDistance > fCvarRadius)
					continue;
				
				g_iIDInflictor[iID] = entity;
				
				ExecuteHamB(Ham_TakeDamage, iID, entity, iIDEntOwner, fCvarDamage - (fCvarDamage / fCvarRadius * fDistance), DMG_BLAST)
				
				g_iIDInflictor[iID] = 0;
			}
			
			engfunc(EngFunc_RemoveEntity, entity)
			
			return HAM_SUPERCEDE;
		}
	}
	
	return HAM_IGNORED;
}
/*
ftClientPrintChatColor(const iIDTarget, const iIDSender = 0, const szMessage[], any:...)
{
	if (iIDTarget && !is_user_connected(iIDTarget))
		return;
	
	static szBuffer[192];
	vformat(szBuffer, charsmax(szBuffer), szMessage, 4)
	
	static const D7_CHAT_TAG[] = "^1[^4D7^1] ";
	format(szBuffer, charsmax(szBuffer), "%s%s", D7_CHAT_TAG, szBuffer)
	
	static iIDMsgSayText;
	if (!iIDMsgSayText)
		iIDMsgSayText = get_user_msgid("SayText");
	
	if (iIDTarget)
		message_begin(MSG_ONE, iIDMsgSayText, _, iIDTarget)
	else
		message_begin(MSG_ALL, iIDMsgSayText)
	
	write_byte(!iIDSender ? iIDTarget : iIDSender)
	write_string(szBuffer)
	message_end()
}*/

// Set entity's rendering type (from fakemeta_util)
stock fm_set_rendering(entity, fx = kRenderFxNone, r = 255, g = 255, b = 255, render = kRenderNormal, amount = 16)
{
	static Float:color[3]
	color[0] = float(r)
	color[1] = float(g)
	color[2] = float(b)
	
	set_pev(entity, pev_renderfx, fx)
	set_pev(entity, pev_rendercolor, color)
	set_pev(entity, pev_rendermode, render)
	set_pev(entity, pev_renderamt, float(amount))
}
