/*
To do list:

[X] dead and/or zombie bit
[X] movetype_follow entity
[X] follow entity glow depending on fuel?
[ ] remove fakemeta or engine
[ ] remove unnecessary code
[ ] optimize
[X] make all the variable names descriptive
[X] upper the customization parts
[ ] cvars for enabling/customizing icon
[X] cs-like drop system
[X] backup and remove trail
[ ] admin menu and/or commands to give/take/drop item
[ ] time-based jetpack (ac/de)celeration & flight(+max thrusts per second)
[ ] cvar to bypass the max thrusts per second if velocity is < than a certain value(default 180.00001)
[ ] cooldown -1 = as soon as the player touches the ground
[ ] find out the player's exact deceleration rate for any gravity level
[ ] fix fuel regen task
[T] fix jetpack disappearing on spawn/new round sometimes
[ ] fix incorrect rendering color?
[ ] fix bots not fully using the fuel
[ ] bypass the acceleration cvar if value is <= 180.0 and velocity is 0
[T] fix icon disappearing on resethud
*/

#include <amxmodx>
#include <engine>
#include <bitsums>
#include <fakemeta>
#include <hamsandwich>
#include <zombieplaguenightmare>

// Uncomment to enable debugging
//#define DefDebug

// Uncomment to draw flame effect
#define DefDrawFlame

// Uncomment to set p_ model
#define DefJetPackPlayerModel

// Uncomment to remove jetpack X time after drop
//#define DefJetPackTimeRemove

const g_iTaskBlockUse = 5000;
const g_iTaskFuelRegenCoolDown = 5001;
const g_iTaskFuelRegenInterval = 5002;
const g_iTaskIconResetHUD = 5003;

//new const g_szEntityClassName[] = "info_target";
new const g_szClassNameJetPack[] = "n4d_jetpack";

#if defined DefJetPackPlayerModel
new const g_szJetPackModelP[] = "models/p_longjump.mdl";
#endif

new const g_szJetPackModelW[] = "models/w_longjump.mdl";
new const g_szJetPackSoundPickUp[] = "items/gunpickup2.wav";
new const g_szJetPackSoundFly[] = "jetpack.wav";
new const g_szJetPackSoundBlow[] = "jp_blow.wav";

#if defined DefDrawFlame
new g_iSpriteFlameID;
#endif

public plugin_precache()
{
	precache_sound(g_szJetPackSoundPickUp)
	precache_sound(g_szJetPackSoundFly)
	precache_sound(g_szJetPackSoundBlow)
	
	#if defined DefDrawFlame
	g_iSpriteFlameID = precache_model("sprites/rjet1.spr"); // rjet1, stmbal1, WXplo1, xsmoke1
	#endif
	
	#if defined DefJetPackPlayerModel
	precache_model(g_szJetPackModelP)
	#endif
	
	precache_model(g_szJetPackModelW)
}

stock colored_chat(const iTarget, const szMessage[], any:...)
{
	if (!is_user_connected(iTarget))
		return;
	
	static const D7_CHAT_TAG[] = "^1[^4ZPNM^1] ";
	
	static szBuffer[192];
	
	// Format message for player
	vformat(szBuffer, 191, szMessage, 3)
	
	format(szBuffer, 191, "%s%s", D7_CHAT_TAG, szBuffer)
	
	static msgSayText;
	if (!msgSayText)
		msgSayText = get_user_msgid("SayText");
	
	message_begin(MSG_ONE, msgSayText, _, iTarget)
	write_byte(iTarget)
	write_string(szBuffer)
	message_end()
}

new g_bitHasJetPack, g_bitZombieOrDead, g_bitIsBot;
new Float:g_fJetPackFuelConsumed[33];
new g_iItemJetPackID, g_bEscapeMap;
new g_pCvarVelocityMax, g_pCvarFuelRegenPercent, g_pCvarFuelRegenCoolDown,
g_pCvarFuelRegenInterval, g_pCvarFuelConsumptionPercent, g_pCvarVelocityAcceleration,
Float:g_fCachedVelocityMax, Float:g_fCachedFuelRegenPercent, Float:g_fCachedFuelRegenCoolDown,
Float:g_fCachedFuelRegenInterval, Float:g_fCachedFuelConsumptionPercent, Float:g_fCachedVelocityAcceleration;

#if defined DefJetPackTimeRemove
new g_pCvarRemoveAfterDrop, Float:g_fCachedRemoveAfterDrop;
#endif

#if defined DefJetPackPlayerModel
new g_iJetPackPlayerEntityID[33];
#endif

public plugin_init()
{
	register_plugin("[ZPNM] JetPack new", "0.3.3", "Bad_Bud, ZmOutStanding, ConnorMcLeod, wbyokomo, D i 5 7 i n c T")
	
	register_event("HLTV", "fw_EventNewRound", "a", "1=0", "2=0")
	register_event("ResetHUD", "fw_EventPlayerResetHUD", "b")
	
	RegisterHam(Ham_Spawn, "player", "fw_HamPlayerSpawn_Post", 1)
	RegisterHam(Ham_Killed, "player", "fw_HamPlayerKilled_Post", 1)
	RegisterHam(Ham_Player_Jump, "player", "fw_HamPlayerJump")
	
	register_touch(g_szClassNameJetPack, "player", "fw_JetPackTouch")
	register_think(g_szClassNameJetPack, "fw_JetPackThink")
	
	g_pCvarVelocityMax = register_cvar("jp_speed", "350.0")
	g_pCvarFuelRegenPercent = register_cvar("jp_regen", "3.757")
	
	#if defined DefJetPackTimeRemove
	g_pCvarRemoveAfterDrop = register_cvar("jp_remove_time", "600.0")
	#endif
	
	g_pCvarFuelRegenCoolDown = register_cvar("jp_cooldown", "5.77")
	g_pCvarFuelRegenInterval = register_cvar("jp_interval", "0.1")
	g_pCvarFuelConsumptionPercent = register_cvar("jp_consumption", "0.5")
	g_pCvarVelocityAcceleration = register_cvar("jp_acceleration", "180.00001")
	
	g_iItemJetPackID = zp_register_extra_item("JetPack", 39, ZP_TEAM_HUMAN | ZP_TEAM_SURVIVOR | ZP_TEAM_SNIPER)
	
	register_clcmd("drop", "CmdDropJP")
	
	new szMapName[32];
	get_mapname(szMapName, 31)
	
	if (containi(szMapName, "escape") != -1 || containi(szMapName, "ze_") != -1 || containi(szMapName, "atix") != -1)
		g_bEscapeMap = true;
}

public fw_EventNewRound()
{
	RemoveAllJetPacks()
	
	g_fCachedVelocityMax = get_pcvar_float(g_pCvarVelocityMax);
	g_fCachedFuelRegenPercent = get_pcvar_float(g_pCvarFuelRegenPercent);
	
	#if defined DefJetPackTimeRemove
	g_fCachedRemoveAfterDrop = get_pcvar_float(g_pCvarRemoveAfterDrop);
	#endif
	
	g_fCachedFuelRegenCoolDown = get_pcvar_float(g_pCvarFuelRegenCoolDown);
	g_fCachedFuelRegenInterval = get_pcvar_float(g_pCvarFuelRegenInterval);
	g_fCachedFuelConsumptionPercent = get_pcvar_float(g_pCvarFuelConsumptionPercent);
	g_fCachedVelocityAcceleration = get_pcvar_float(g_pCvarVelocityAcceleration);
}

public client_putinserver(id)
{
	bitsum_add(g_bitZombieOrDead, id)
	
	if (is_user_bot(id))
		bitsum_add(g_bitIsBot, id)
}

public fw_EventPlayerResetHUD(id)
{
	if (!bitsum_get(g_bitHasJetPack, id))
		return;
	
	set_task(0.1, "ComputeIconColor", id + g_iTaskIconResetHUD)
}

public zpnm_register_ham_czbots(id)
{
	RegisterHamFromEntity(Ham_Spawn, id, "fw_HamPlayerSpawn_Post", 1)
	RegisterHamFromEntity(Ham_Killed, id, "fw_HamPlayerKilled_Post", 1)
	RegisterHamFromEntity(Ham_Player_Jump, id, "fw_HamPlayerJump")
	RegisterHamFromEntity(Ham_TakeDamage, id, "fw_HamPlayerTakeDamage")
}

public fw_HamPlayerTakeDamage(id, inflictor, attacker, Float:damage, damage_type)
{
	// Prevent from taking fall damage when having a jetpack
	if (!bitsum_get(g_bitHasJetPack, id) || !(damage_type & (1<<5)))
		return HAM_IGNORED;
	
	return HAM_SUPERCEDE;
}

public client_disconnect(id)
{
	DropJetPack(id)
	
	g_fJetPackFuelConsumed[id] = 0.0;
	
	#if defined DefJetPackPlayerModel
	g_iJetPackPlayerEntityID[id] = 0;
	#endif
	
	bitsum_del(g_bitIsBot, id)
}

public fw_HamPlayerSpawn_Post(id)
{
	if (!is_user_alive(id))
		return;
	
}

public zpnm_user_spawn_post(id)
{
	if (bitsum_get(g_bitIsBot, id))
		zp_force_buy_extra_item(id, g_iItemJetPackID)
	
	bitsum_del(g_bitZombieOrDead, id)
	
	if (g_fJetPackFuelConsumed[id] == -1.0)
		bitsum_add(g_bitHasJetPack, id)
	else if (!bitsum_get(g_bitHasJetPack, id))
		return;
	
	remove_task(id + g_iTaskFuelRegenCoolDown)
	remove_task(id + g_iTaskFuelRegenInterval)
	remove_task(id + g_iTaskBlockUse)
	
	ProgressBar(id, 0)
	
	g_fJetPackFuelConsumed[id] = 0.0;
	
	IconJetPackFuel(id)
/*	
	#if defined DefJetPackPlayerModel
	g_iJetPackPlayerEntityID[id] = create_entity("info_target");
	
	if (is_valid_ent(g_iJetPackPlayerEntityID[id]))
		fw_JetPackTouch(g_iJetPackPlayerEntityID[id], id)
	#endif
	*/
}

public zp_extra_item_selected(id, item)
{
	if (item != g_iItemJetPackID)
		return PLUGIN_CONTINUE;
	
	if (g_bEscapeMap && !(get_user_flags(id) & ADMIN_RCON))
	{
		colored_chat(id, "^3You can't buy jetpack on escape maps^1!")
		
		return ZP_PLUGIN_HANDLED;
	}
	else if (bitsum_get(g_bitHasJetPack, id))
	{
		colored_chat(id, "^3You already have a jetpack^1!")
		
		return ZP_PLUGIN_HANDLED;
	}
	
	bitsum_add(g_bitHasJetPack, id)
	
	remove_task(id + g_iTaskFuelRegenCoolDown)
	remove_task(id + g_iTaskFuelRegenInterval)
	remove_task(id + g_iTaskBlockUse)
	
	g_fJetPackFuelConsumed[id] = 0.0;
	
	IconJetPackFuel(id)
	
	emit_sound(id, CHAN_STATIC, g_szJetPackSoundPickUp, VOL_NORM, ATTN_NORM, 0, PITCH_NORM)
	
	#if defined DefJetPackPlayerModel
	g_iJetPackPlayerEntityID[id] = create_entity("info_target");
	
	if (is_valid_ent(g_iJetPackPlayerEntityID[id]))
	{
		engfunc(EngFunc_SetModel, g_iJetPackPlayerEntityID[id], g_szJetPackModelP)
		set_pev(g_iJetPackPlayerEntityID[id], pev_movetype, MOVETYPE_FOLLOW)
		set_pev(g_iJetPackPlayerEntityID[id], pev_aiment, id)
		set_pev(g_iJetPackPlayerEntityID[id], pev_solid, SOLID_NOT)
		set_pev(g_iJetPackPlayerEntityID[id], pev_owner, id)
		set_pev(g_iJetPackPlayerEntityID[id], pev_rendermode, kRenderNormal)
		set_pev(g_iJetPackPlayerEntityID[id], pev_renderfx, kRenderFxGlowShell)
		set_pev(g_iJetPackPlayerEntityID[id], pev_rendercolor, Float:{ 0.0, 255.0, 0.0 })
		set_pev(g_iJetPackPlayerEntityID[id], pev_renderamt, 5.77) //random_float(57.7, 75.7)
	}
	#endif
	
	colored_chat(id, "JetPack usage: ^4JUMP^1 + ^4DUCK^1 = ^4Fly^1 | Loading bar = Cooldown BEFORE refuel regen start")
	
	return PLUGIN_CONTINUE;
}

public zp_user_humanized_post(id)
{
//	g_fJetPackFuelConsumed[id] = 0.0;
	bitsum_del(g_bitZombieOrDead, id)
	
	if (bitsum_get(g_bitIsBot, id))
		zp_force_buy_extra_item(id, g_iItemJetPackID)
}

public CmdDropJP(id)
{
	if (!bitsum_get(g_bitHasJetPack, id) || bitsum_get(g_bitZombieOrDead, id) || get_user_weapon(id) != CSW_KNIFE)
		return PLUGIN_CONTINUE;
	
	DropJetPack(id)
	
	return PLUGIN_HANDLED;
}

public zp_user_infected_post(id)
{
	bitsum_add(g_bitZombieOrDead, id)
	
	g_fJetPackFuelConsumed[id] = 0.0;
	
	DropJetPack(id);
}

public fw_JetPackTouch(iEnt, id)
{
	if (bitsum_get(g_bitHasJetPack, id) || bitsum_get(g_bitZombieOrDead, id) || pev(iEnt, pev_iuser4) == 11111)
		return;
	
	#if !defined DefJetPackPlayerModel
	engfunc(EngFunc_RemoveEntity, iEnt)
	#else
	g_iJetPackPlayerEntityID[id] = iEnt;
//	g_iJetPackPlayerEntityID[pev(iEnt, pev_owner)] = 0;
	
	engfunc(EngFunc_SetModel, g_iJetPackPlayerEntityID[id], g_szJetPackModelP)
	set_pev(g_iJetPackPlayerEntityID[id], pev_classname, "")
	set_pev(g_iJetPackPlayerEntityID[id], pev_movetype, MOVETYPE_FOLLOW)
	set_pev(g_iJetPackPlayerEntityID[id], pev_aiment, id)
	set_pev(g_iJetPackPlayerEntityID[id], pev_solid, SOLID_NOT)
	set_pev(g_iJetPackPlayerEntityID[id], pev_owner, id)
//	set_pev(g_iJetPackPlayerEntityID[id], pev_rendermode, kRenderNormal)
//	set_pev(g_iJetPackPlayerEntityID[id], pev_renderfx, kRenderFxGlowShell)
//	set_pev(g_iJetPackPlayerEntityID[id], pev_rendercolor, Float:{ 0.0, 255.0, 0.0 })
//	set_pev(g_iJetPackPlayerEntityID[id], pev_renderamt, random_float(57.7, 75.7))
	#endif
	
	bitsum_add(g_bitHasJetPack, id)
	
	ComputeIconColor(id + g_iTaskIconResetHUD)
	
	emit_sound(id, CHAN_STATIC, g_szJetPackSoundPickUp, VOL_NORM, ATTN_NORM, 0, PITCH_NORM)
	
	colored_chat(id, "JetPack usage: ^4JUMP^1 + ^4DUCK^1 = ^4Fly^1 | Loading bar = Cooldown BEFORE refuel start")
}

public fw_HamPlayerKilled_Post(id)
{
	bitsum_add(g_bitZombieOrDead, id)
	
	// Only lose the item permanently if the round has started
	if (zp_has_round_started())
		g_fJetPackFuelConsumed[id] = 0.0;
	else
		g_fJetPackFuelConsumed[id] = -1.0;
	
	DropJetPack(id)
}

DropJetPack(const id)
{
	if (!bitsum_get(g_bitHasJetPack, id))
		return;
	
	bitsum_del(g_bitHasJetPack, id)
	
	remove_task(id + g_iTaskBlockUse)
	
	
	IconJetPackFuel(id, _, 0)
	
	if (g_fJetPackFuelConsumed[id] == -1.0)
	{
		if (task_exists(id + g_iTaskFuelRegenCoolDown))
		{
			ProgressBar(id, 0)
			
			remove_task(id + g_iTaskFuelRegenCoolDown)
		}
		
		remove_task(id + g_iTaskFuelRegenInterval)
		
		return;
	}
	
	#if defined DefJetPackPlayerModel
	new iEnt = g_iJetPackPlayerEntityID[id];
	set_pev(iEnt, pev_aiment, 0)
	g_iJetPackPlayerEntityID[id] = 0;
	#else
	new iEnt = create_entity("info_target"); //engfunc(EngFunc_CreateEntity, engfunc(EngFunc_AllocString, "info_target")) g_szEntityClassName
	
	if (!iEnt || !is_valid_ent(iEnt)) //pev_valid(iEnt)
		return;
	#endif
	
	set_pev(iEnt, pev_classname, g_szClassNameJetPack)
	set_pev(iEnt, pev_solid, SOLID_TRIGGER)
	set_pev(iEnt, pev_movetype, MOVETYPE_TOSS)
	engfunc(EngFunc_SetModel, iEnt, g_szJetPackModelW)
//	engfunc(EngFunc_SetSize, iEnt, Float:{ -5.0, -5.0, -5.0 }, Float:{ 5.0, 5.0, 5.0 })
	new Float:fOrigin[3], Float:fVelocity[3];
//	fOrigin[2] += 5.7
//	engfunc(EngFunc_SetOrigin, iEnt, fOrigin)
	
	pev(id, pev_origin, fOrigin)
	set_pev(iEnt, pev_origin, fOrigin)
	
	pev(id, pev_angles, fOrigin)
	fOrigin[0] = 0.0;
	set_pev(iEnt, pev_angles, fOrigin)
	
	velocity_by_aim(id, 250, fOrigin)
	pev(id, pev_velocity, fVelocity)
	
	fVelocity[0] += fOrigin[0];
	fVelocity[1] += fOrigin[1];
	fVelocity[2] += fOrigin[2];
	
	set_pev(iEnt, pev_velocity, fVelocity)
//	set_pev(iEnt, pev_effects, EF_LIGHT)
	set_pev(iEnt, pev_iuser4, 11111)
	set_pev(iEnt, pev_flTimeStepSound, 200)
	set_pev(iEnt, pev_nextthink, get_gametime() + 0.57) // prevent drop/pickup exploit
/*	
	fOrigin[0] = random_float(57.7, 255.0);
	fOrigin[1] = random_float(0.0, 255.0);
	fOrigin[2] = random_float(0.0, 255.0);
	
	set_pev(iEnt, pev_rendercolor, fOrigin)
	set_pev(iEnt, pev_rendermode, kRenderNormal) //kRenderTransAdd
	set_pev(iEnt, pev_renderfx, kRenderFxGlowShell)
	set_pev(iEnt, pev_renderamt, random_float(57.7, 75.7))*/
}

public client_PreThink(id)
{
/*	if (bitsum_get(g_bitHasJetPack, id) && !(pev(id, pev_flags) & FL_ONGROUND) && !(pev(id, pev_flags) & FL_FLY))
	{
		static fVelocity[3];
		pev(id, pev_velocity, fVelocity)
		
		client_print(id, print_chat, "Velocity: %.2f PreThink", fVelocity[2])
		
		return;
	}
	else */if (!bitsum_get(g_bitIsBot, id) || !bitsum_get(g_bitHasJetPack, id) || task_exists(id + g_iTaskFuelRegenInterval))
		return;
	
	set_pev(id, pev_button, pev(id, pev_button) | IN_DUCK)
	fw_HamPlayerJump(id)
}

public fw_HamPlayerJump(id)
{
	if (!bitsum_get(g_bitHasJetPack, id)/* || task_exists(id + g_iTaskBlockUse)*/)
		return;
	
//	static Float:fVelocityUpToReach[33];
	
	if (g_fJetPackFuelConsumed[id] == 100.0)
	{
//		fVelocityUpToReach[id] = 0.0;
		
		return;
	}
	
/*	if (pev(id, pev_flags) & FL_ONGROUND)
		fVelocityUpToReach[id] = 0.0;
	*/
	new iBitSumButtons = pev(id, pev_button);
	
	if (!(iBitSumButtons & IN_DUCK))/* || !(~pev(id, pev_flags) & FL_ONGROUND)*/
		return;
	
	static Float:fVelocity[3]/*, Float:fVelocityUpLast[33]*/;
	pev(id, pev_velocity, fVelocity)
	
	#if defined DefDebug
	log_to_file("JetPack.log", "[HamPlayerJump] Player %d. Current velocity: %.2f.", id, fVelocity[2])
	
	log_to_file("JetPack.log", "[HamPlayerJump] Player %d. Current fuel consumption percentage: %.2f.", id, g_fJetPackFuelConsumed[id])
	#endif
	
	// We're already at or above the max vertical velocity
	// Skip this frame and save fuel
	if (fVelocity[2] >= g_fCachedVelocityMax)
	{
		#if defined DefDebug
		log_to_file("JetPack.log", "[HamPlayerJump] Player %d. Current velocity is higher or equal to the max velocity(%.2f).", id, g_fCachedVelocityMax)
		log_to_file("JetPack.log", "")
		#endif
		
		return;
	}
	
/*	if (fVelocityUpToReach[id] == 0.0)
		fVelocityUpToReach[id] += fVelocity[2];
	
	fVelocityUpToReach[id] += g_fCachedVelocityAcceleration;
	
	if (fVelocityUpToReach[id] > g_fCachedVelocityMax)
		fVelocityUpToReach[id] = g_fCachedVelocityMax;
	*/
	static Float:fVelocityUpNew;
	fVelocityUpNew = fVelocity[2] + g_fCachedVelocityAcceleration;
	
	if (fVelocityUpNew > g_fCachedVelocityMax)
		fVelocityUpNew = g_fCachedVelocityMax;
	
	if (iBitSumButtons & IN_FORWARD)
		velocity_by_aim(id, floatround(g_fCachedVelocityMax), fVelocity)
	else if (iBitSumButtons & IN_BACK)
		velocity_by_aim(id, -floatround(g_fCachedVelocityMax), fVelocity)
	
	fVelocity[2] = fVelocityUpNew/*fVelocityUpToReach[id]*/;
	
	set_pev(id, pev_velocity, fVelocity)
	
	#if defined DefDebug
	log_to_file("JetPack.log", "[HamPlayerJump] Player %d. New velocity: %.2f.", id, fVelocity[2])
	#endif
	
//	fVelocityUpToReach[id] = fVelocityUpNew;
	
	#if defined DefDrawFlame
	pev(id, pev_origin, fVelocity)
	
	engfunc(EngFunc_MessageBegin, MSG_PVS, SVC_TEMPENTITY, fVelocity, 0)
	write_byte(TE_SPRITE)
	engfunc(EngFunc_WriteCoord, fVelocity[0])
	engfunc(EngFunc_WriteCoord, fVelocity[1])
	engfunc(EngFunc_WriteCoord, fVelocity[2])
	write_short(g_iSpriteFlameID)
	write_byte(8)
	write_byte(128)
	message_end()
	#endif
	
	g_fJetPackFuelConsumed[id] += g_fCachedFuelConsumptionPercent;
	
	if (g_fJetPackFuelConsumed[id] > 100.0)
		g_fJetPackFuelConsumed[id] = 100.0;
	
	#if defined DefDebug
	log_to_file("JetPack.log", "[HamPlayerJump] Player %d. New fuel consumption percentage: %.2f.", id, g_fJetPackFuelConsumed[id])
	#endif
	
	set_task(0.1, "Task_BlockUse", id + g_iTaskBlockUse)
	
	remove_task(id + g_iTaskFuelRegenCoolDown)
	remove_task(id + g_iTaskFuelRegenInterval)
	
	ComputeIconColor(id + g_iTaskIconResetHUD)
	
	set_task(g_fCachedFuelRegenCoolDown, "Task_FuelRegenStart", id + g_iTaskFuelRegenCoolDown)
	
	ProgressBar(id, floatround(g_fCachedFuelRegenCoolDown))
	
	if (g_fJetPackFuelConsumed[id] <= 75.0)
		emit_sound(id, CHAN_STREAM, g_szJetPackSoundFly, VOL_NORM, ATTN_NORM, 0, PITCH_NORM)
	else
		emit_sound(id, CHAN_STREAM, g_szJetPackSoundBlow, VOL_NORM, ATTN_NORM, 0, PITCH_NORM);
	
	#if defined DefDebug
	log_to_file("JetPack.log", "")
	#endif
}

public Task_BlockUse()
{
	/* Do nothing */
}

public Task_FuelRegenStart(iID)
{
	iID -= g_iTaskFuelRegenCoolDown;
	
	Task_FuelRegen(iID + g_iTaskFuelRegenInterval)
	set_task(g_fCachedFuelRegenInterval, "Task_FuelRegen", iID + g_iTaskFuelRegenInterval, _, _, "b")
	
	ProgressBar(iID, 1, 100)
}

public Task_FuelRegen(iID)
{
	iID -= g_iTaskFuelRegenInterval;
	
	g_fJetPackFuelConsumed[iID] -= g_fCachedFuelRegenPercent;
	
	if (g_fJetPackFuelConsumed[iID] <= 0.0)
	{
		g_fJetPackFuelConsumed[iID] = 0.0;
		
		IconJetPackFuel(iID, { 0, 255, 0 }, 1)
		
		remove_task(iID + g_iTaskFuelRegenInterval)
	}
	else
	{
		if (bitsum_get(g_bitHasJetPack, iID))
		{
	//		client_print(iID, print_center, "JetPack fuel: %.2f", g_fJetPackFuelConsumed[iID])
			
			ComputeIconColor(iID + g_iTaskIconResetHUD)
		}
	}
}

public ComputeIconColor(iID)
{
	iID -= g_iTaskIconResetHUD;
	
	if (g_fJetPackFuelConsumed[iID] == 100.0)
	{
		IconJetPackFuel(iID, { 157, 157, 157 }, !task_exists(iID + g_iTaskFuelRegenInterval) ? 1 : 2)
		
		#if defined DefJetPackPlayerModel
		set_pev(g_iJetPackPlayerEntityID[iID], pev_rendercolor, Float:{ 157.0, 157.0, 157.0})
		#endif
		
		// 100% consumed(tank is empty)
		return;
	}
	
	static Float:fColorAmounts[3], iColorAmounts[3];
	
	fColorAmounts[0] = 255 * g_fJetPackFuelConsumed[iID] / 100; // red
	fColorAmounts[1] = 255 - fColorAmounts[0]; // green
	
	#if defined DefJetPackPlayerModel
	set_pev(g_iJetPackPlayerEntityID[iID], pev_rendercolor, fColorAmounts)
	#endif
	
	iColorAmounts[0] = floatround(fColorAmounts[0]);
	iColorAmounts[1] = floatround(fColorAmounts[1]);
	
	IconJetPackFuel(iID, iColorAmounts, !task_exists(iID + g_iTaskFuelRegenInterval) ? 1 : 2)
}

stock IconJetPackFuel(const iID, const iColorAmounts[3] = { 0, 255, 0 }, const iMode = 1)
{
	static msgStatusIcon;
	if (!msgStatusIcon)
		msgStatusIcon = get_user_msgid("StatusIcon");
	
	message_begin(MSG_ONE, msgStatusIcon, _, iID)
	write_byte(iMode) // status (0=hide, 1=show, 2=flash)
	write_string("item_longjump") // sprite name
	write_byte(iColorAmounts[0])
	write_byte(iColorAmounts[1])
	write_byte(iColorAmounts[2])
	message_end()
}

stock ProgressBar(const iID, const iTime, const iStartPercent = 0)
{
	static msgBarTime2;
	if (!msgBarTime2)
		msgBarTime2 = get_user_msgid("BarTime2");
	
	message_begin(MSG_ONE, msgBarTime2, _, iID)
	write_short(iTime)
	write_short(iStartPercent)
	message_end()
}

public fw_JetPackThink(iEnt)
{
	if (pev_valid(iEnt) != 2)
		return;
	
	switch (pev(iEnt, pev_flTimeStepSound))
	{
		case 200:
		{
			set_pev(iEnt, pev_iuser4, 22222)
			
			#if defined DefJetPackTimeRemove
			set_pev(iEnt, pev_flTimeStepSound, 201)
			set_pev(iEnt, pev_nextthink, get_gametime() + g_fCachedRemoveAfterDrop)
			#endif
		}
		#if defined DefJetPackTimeRemove
		case 201: engfunc(EngFunc_RemoveEntity, iEnt)
		#endif
	}
}

RemoveAllJetPacks()
{
	new iEnt;
	
	while ((iEnt = engfunc(EngFunc_FindEntityByString, -1, "classname", g_szClassNameJetPack)) > 0)
		engfunc(EngFunc_RemoveEntity, iEnt)
}
