/*
To do list:

[X] dead and/or zombie bit
[X] movetype_follow entity
[X] follow entity glow depending on fuel?
[ ] remove fakemeta or engine
[ ] remove unnecessary code
[ ] optimize
[X] make all the variable names descriptive
[X] upper the customization parts
[ ] cvars for enabling/customizing icon
[ ] cvars for enabling/customizing player model
[X] cs-like drop system
[ ] admin menu and/or commands to give/take/drop item
[ ] time-based jetpack (ac/de)celeration
[X] time-based flight(+max thrusts per second)
[ ] cvar to bypass the max thrusts per second if velocity is < than a certain value(default 180.00001)
[ ] cooldown -1 = as soon as the player touches the ground
[ ] find out the player's exact deceleration rate for any gravity level
[X] fix fuel regen task
[X] fix jetpack disappearing on spawn/new round sometimes
[X] fix incorrect rendering color?
[X] fix bots not fully using the fuel
[T] bypass the acceleration cvar if value is <= 180.0 and velocity is 0.0
[X] fix icon disappearing on resethud
[X] custom keys to press
[ ] trail instead of sprite message
[X] (re)pick up delay
[X] (re)pick up delay cvar
[-] velocity by aim remove X vector?
[ ] fix velocity_by_aim messing with our vertical velocity
[T] fixed bug where dead players could fly with jetpack after not fully losing it on death(mode not started)
[T] fixed bug where enable pickup task would remain active after entity was removed after drop
[T] added removing the update hud task wherever it was necessary
[ ] custom touch forward
[T] fixed bug where entity id wouldn't be removed from the jetpack list array when entity was removed after drop
[T] changed the function which was removing all of the jetpacks to be able to remove only a single specified jetpack too
[ ] initialize all information on entity create
[ ] delete all information on entity remove?
[T] fix jetpacks getting removed from players which died before round mode start
*/

#include <amxmodx>
#include <engine>
#include <bitsums>
#include <fakemeta>
#include <hamsandwich>
#include <zombieplaguenightmare>

// Uncomment to enable debugging
//#define DefDebug

#if defined DefDebug
new g_szFileDebug[] = "D7_JetPack.log";
#endif

//new const g_szEntityClassName[] = "info_target";
new const g_szClassNameJetPack[] = "n4d_jetpack";

new const g_szJetPackModelP[] = "models/p_longjump.mdl";
new const g_szJetPackModelW[] = "models/w_longjump.mdl";
new const g_szJetPackSpriteFlame[] = "sprites/rjet1.spr"; // rjet1, stmbal1, WXplo1, xsmoke1
new const g_szJetPackSoundPickUp[] = "items/gunpickup2.wav";
new const g_szJetPackSoundFly[] = "jetpack.wav";
new const g_szJetPackSoundBlow[] = "jp_blow.wav";

new g_iSpriteFlameID;

new Array:g_ArrayJetPackEntityList;

public plugin_precache()
{
	g_iSpriteFlameID = precache_model(g_szJetPackSpriteFlame);
	
	g_ArrayJetPackEntityList = ArrayCreate(1, 1);
	
	precache_sound(g_szJetPackSoundPickUp)
	precache_sound(g_szJetPackSoundFly)
	precache_sound(g_szJetPackSoundBlow)
	
	precache_model(g_szJetPackModelP)
	precache_model(g_szJetPackModelW)
}

colored_chat(const iTarget, const szMessage[], any:...)
{
	if (!is_user_connected(iTarget))
		return;
	
	static const D7_CHAT_TAG[] = "^1[^4ZPNM^1] ";
	
	static szBuffer[192];
	
	// Format message for player
	vformat(szBuffer, 191, szMessage, 3)
	
	format(szBuffer, 191, "%s%s", D7_CHAT_TAG, szBuffer)
	
	static msgSayText;
	if (!msgSayText)
		msgSayText = get_user_msgid("SayText");
	
	message_begin(MSG_ONE, msgSayText, _, iTarget)
	write_byte(iTarget)
	write_string(szBuffer)
	message_end()
}

new g_iJetPackEntityID[33];
new /*g_bitHoldingButtons, */g_bitZombieOrDead, g_bitIsBot;

const g_fFuelConsumed = pev_fuser1;
const g_fNextThrust = pev_fuser2;
const g_fNextRefuel = pev_fuser3;
const g_fThrottleVelocity = pev_fuser4; //g_fNextAcceleration
const g_bIsRefueling = pev_iuser2;
//const g_fNextDeceleration = pev_iuser3;
//const g_fThrottleVelocity = pev_iuser4;

const g_iTaskEntityEnablePickUpID = 5000;
const g_iTaskEntityRemoveID = 5001;

new g_iItemJetPackID, g_bEscapeMap;
new g_pCvarVelocityMax, g_pCvarFuelRegenPercent, g_pCvarFuelRegenCoolDown,
g_pCvarFuelRegenInterval, g_pCvarFuelConsumptionPercent, g_pCvarVelocityAcceleration,
Float:g_fCachedVelocityMax, Float:g_fCachedFuelRegenPercent, Float:g_fCachedFuelRegenCoolDown,
Float:g_fCachedFuelRegenInterval, Float:g_fCachedFuelConsumptionPercent, Float:g_fCachedVelocityAcceleration;

new g_pCvarRemoveAfterDrop, g_pCvarThrustMinimumDelay, g_pCvarButtons, g_pCvarPickUpCoolDown;

public plugin_init()
{
	register_plugin("[ZPNM] JetPack new", "0.3.7", "D i 5 7 i n c T, Bad_Bud, ZmOutStanding, ConnorMcLeod, wbyokomo, NiHiLaNTh")
	
	register_event("HLTV", "fw_EventNewRound", "a", "1=0", "2=0")
	register_event("ResetHUD", "fw_EventPlayerResetHUD", "b")
	
	RegisterHam(Ham_Spawn, "player", "fw_HamPlayerSpawn_Post", 1)
	RegisterHam(Ham_Killed, "player", "fw_HamPlayerKilled_Post", 1)
//	RegisterHam(Ham_Player_Jump, "player", "fw_HamPlayerJump")
	//RegisterHam(Ham_TakeDamage, "player", "fw_HamPlayerTakeDamage")
	
	register_touch(g_szClassNameJetPack, "player", "fw_JetPackTouch")
//	register_think(g_szClassNameJetPack, "fw_JetPackThink")
	
	g_pCvarVelocityMax = register_cvar("jp_speed", "265.0")
	g_pCvarFuelRegenPercent = register_cvar("jp_regen", "3.757")
	
	g_pCvarRemoveAfterDrop = register_cvar("jp_remove_time", "600.0")
	
	g_pCvarFuelRegenCoolDown = register_cvar("jp_cooldown", "5.77")
	g_pCvarFuelRegenInterval = register_cvar("jp_interval", "0.1")
	g_pCvarFuelConsumptionPercent = register_cvar("jp_consumption", "1.0")
	g_pCvarVelocityAcceleration = register_cvar("jp_acceleration", "50.0")
	g_pCvarThrustMinimumDelay = register_cvar("jp_thrust_minimum_delay", "0.05")
	g_pCvarButtons = register_cvar("jp_buttons", "6")
	g_pCvarPickUpCoolDown = register_cvar("jp_pickup_cooldown", "0.57")
	
	g_iItemJetPackID = zp_register_extra_item("JetPack", 39, ZP_TEAM_HUMAN | ZP_TEAM_SURVIVOR | ZP_TEAM_SNIPER)
	
	register_clcmd("drop", "fw_ClCmdDrop")
	
	new szMapName[32];
	get_mapname(szMapName, 31)
	
	if (containi(szMapName, "escape") != -1 || containi(szMapName, "ze_") != -1 || containi(szMapName, "atix") != -1)
		g_bEscapeMap = true;
}

public server_frame()
{
	new iArraySizeJetPackList = ArraySize(g_ArrayJetPackEntityList);
/*	
	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Server frame] Found %d JetPack entites.", iArraySizeJetPackList)
	#endif
	*/
	if (!iArraySizeJetPackList)
		return;
	
	new Float:fGameTime = get_gametime(), Float:fThrustMinimumDelay = get_pcvar_float(g_pCvarThrustMinimumDelay);
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Server frame] Game time: %f. Thrust minimum delay: %f.", fGameTime, fThrustMinimumDelay)
	#endif
	*/
	for (new iArrayPosition; iArrayPosition < iArraySizeJetPackList; iArrayPosition++)
	{
/*		#if defined DefDebug
		log_to_file(g_szFileDebug, "[Server frame] Found jetpack entity: %d.", iEntityID)
		#endif
		*/
		fwJetPackEntityThink(ArrayGetCell(g_ArrayJetPackEntityList, iArrayPosition), fGameTime, fThrustMinimumDelay)
	}
/*	
	#if defined DefDebug
	log_to_file(g_szFileDebug, "")
	#endif*/
}

fwJetPackEntityThink(const iEntityID, const Float:fGameTime, const Float:fThrustMinimumDelay)
{
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[JetPack think] Game time: %f | User: %d | Buttons: %d | Refuel time left: %f | Fuel consumed: %f.",
	fGameTime, iUserID, iBitSumButtons, fNextRefuel, fFuelConsumed)
	#endif
	*/
	new iUserID = pev(iEntityID, pev_iuser1), Float:fFuelConsumed, Float:fNextRefuel;
	
	pev(iEntityID, g_fFuelConsumed, fFuelConsumed)
	
	new Float:fNextThrust;
	pev(iEntityID, g_fNextThrust, fNextThrust)
	
	if (fNextThrust <= fGameTime)
	{
		if (iUserID && iUserID < 33 && fFuelConsumed < 100.0)
		{
			new iBitSumButtons = pev(iUserID, pev_button), iBitSumButtonsCVar = get_pcvar_num(g_pCvarButtons);
			
			if ((iBitSumButtons & iBitSumButtonsCVar) == iBitSumButtonsCVar || bitsum_get(g_bitIsBot, iUserID) && !pev(iEntityID, g_bIsRefueling))
			{
				static Float:fVelocity[3];
				pev(iUserID, pev_velocity, fVelocity)
				
				#if defined DefDebug
				if (!bitsum_get(g_bitIsBot, iUserID))
					log_to_file(g_szFileDebug, "[JetPack think] Player current vertical velocity: %f.", fVelocity[2])
				#endif
				
				if (fVelocity[2] < g_fCachedVelocityMax)
				{
/*					#if defined DefDebug
					log_to_file(g_szFileDebug, "[JetPack think] The current vertical velocity is above or equal to the max(%f). Skipping this frame.", g_fCachedVelocityMax)
					#endif
					*/
					new Float:fVelocityUpNew = fVelocity[2]/* + ((fVelocity[2] != 0.0) ? g_fCachedVelocityAcceleration : 181.0)*/;
					
					if (iBitSumButtons & IN_FORWARD)
						velocity_by_aim(iUserID, floatround(g_fCachedVelocityMax), fVelocity)
					else if (iBitSumButtons & IN_BACK)
						velocity_by_aim(iUserID, -floatround(g_fCachedVelocityMax), fVelocity)
					
					pev(iEntityID, g_fThrottleVelocity, fVelocity[2])
					
					fVelocity[2] += g_fCachedVelocityAcceleration;
					
					set_pev(iEntityID, g_fThrottleVelocity, fVelocity[2])
					
					fVelocity[2] += fVelocityUpNew;
					
					if (fVelocity[2] > g_fCachedVelocityMax)
						fVelocity[2] = g_fCachedVelocityMax;
					
					#if defined DefDebug
					if (!bitsum_get(g_bitIsBot, iUserID))
						log_to_file(g_szFileDebug, "[JetPack think] New vertical velocity: %f.", fVelocity[2])
					#endif
					
					set_pev(iUserID, pev_velocity, fVelocity)
					
					pev(iUserID, pev_origin, fVelocity)
					
					engfunc(EngFunc_MessageBegin, MSG_PVS, SVC_TEMPENTITY, fVelocity, 0)
					write_byte(TE_SPRITE)
					engfunc(EngFunc_WriteCoord, fVelocity[0])
					engfunc(EngFunc_WriteCoord, fVelocity[1])
					engfunc(EngFunc_WriteCoord, fVelocity[2])
					write_short(g_iSpriteFlameID)
					write_byte(8) //scale in 0.1's
					write_byte(157) //brightness
					message_end()
					
					fFuelConsumed += g_fCachedFuelConsumptionPercent;
					
					if (fFuelConsumed >= 100.0)
						fFuelConsumed = 100.0;
					else
						set_pev(iEntityID, g_fNextThrust, fGameTime + fThrustMinimumDelay)
					
					#if defined DefDebug
					if (!bitsum_get(g_bitIsBot, iUserID))
						log_to_file(g_szFileDebug, "[JetPack think] New fuel consumption percentage: %f.", fFuelConsumed)
					#endif
					
					set_pev(iEntityID, g_fFuelConsumed, fFuelConsumed)
					
					#if defined DefDebug
					if (!bitsum_get(g_bitIsBot, iUserID))
						log_to_file(g_szFileDebug, "[JetPack think] New thrust will be available in %f seconds.", fThrustMinimumDelay)
					#endif
					
					ComputeColor(iEntityID, fFuelConsumed, iUserID, false)
					
					fNextRefuel = fGameTime + g_fCachedFuelRegenCoolDown;
					set_pev(iEntityID, g_fNextRefuel, fNextRefuel)
					set_pev(iEntityID, g_bIsRefueling, false)
					
					#if defined DefDebug
					if (!bitsum_get(g_bitIsBot, iUserID))
						log_to_file(g_szFileDebug, "[JetPack think] New refuel will be available in %f seconds.", g_fCachedFuelRegenCoolDown)
					#endif
					
					ProgressBar(iUserID, floatround(g_fCachedFuelRegenCoolDown))
					
					emit_sound(iEntityID, CHAN_STREAM, (fFuelConsumed <= 75.0) ? g_szJetPackSoundFly : g_szJetPackSoundBlow, VOL_NORM, ATTN_NORM, 0, PITCH_NORM)
				}
			}
		}
		
		// Hasn't just been used
		if (fNextRefuel == 0.0)
		{
			pev(iEntityID, g_fThrottleVelocity, fNextThrust)
			
			if (fNextThrust > 0.0)
			{
				fNextThrust -= g_fCachedVelocityAcceleration;
				
				if (fNextThrust < 0.0)
					fNextThrust = 0.0;
				
				set_pev(iEntityID, g_fThrottleVelocity, fNextThrust)
			}
		}
	}
	
	if (fFuelConsumed > 0.0 && fNextRefuel == 0.0)
	{
		pev(iEntityID, g_fNextRefuel, fNextRefuel)
		
		if (fNextRefuel <= fGameTime)
		{
			fNextRefuel -= fGameTime;
			
/*			#if defined DefDebug
			log_to_file(g_szFileDebug, "[JetPack think] It's time for the next refuel. Time left: %f.", fNextRefuel)
			log_to_file(g_szFileDebug, "[JetPack think] Current fuel consumption percentage: %f.", fFuelConsumed)
			#endif
			*/
			fFuelConsumed -= g_fCachedFuelRegenPercent;
			
			if (fFuelConsumed <= 0.0)
			{
				fFuelConsumed = 0.0;
				set_pev(iEntityID, g_fNextRefuel, 0.0)
				set_pev(iEntityID, g_bIsRefueling, false)
				ComputeColor(iEntityID, fFuelConsumed, iUserID, false)
			}
			else
			{
				set_pev(iEntityID, g_fNextRefuel, fGameTime + fNextRefuel + g_fCachedFuelRegenInterval)
				set_pev(iEntityID, g_bIsRefueling, true)
				ComputeColor(iEntityID, fFuelConsumed, iUserID, true)
			}
			
			set_pev(iEntityID, g_fFuelConsumed, fFuelConsumed)
			
			#if defined DefDebug
			log_to_file(g_szFileDebug, "[JetPack think] New fuel consumption percentage: %f. Next refuel in %f seconds.", fFuelConsumed, fGameTime + fNextRefuel + g_fCachedFuelRegenInterval)
			#endif
		}
	}
}

public fw_EventNewRound()
{
	RemoveJetPack()
	
	g_fCachedVelocityMax = get_pcvar_float(g_pCvarVelocityMax);
	g_fCachedFuelRegenPercent = get_pcvar_float(g_pCvarFuelRegenPercent);
	
	g_fCachedFuelRegenCoolDown = get_pcvar_float(g_pCvarFuelRegenCoolDown);
	g_fCachedFuelRegenInterval = get_pcvar_float(g_pCvarFuelRegenInterval);
	g_fCachedFuelConsumptionPercent = get_pcvar_float(g_pCvarFuelConsumptionPercent);
	g_fCachedVelocityAcceleration = get_pcvar_float(g_pCvarVelocityAcceleration);
}

public client_putinserver(id)
{
	bitsum_add(g_bitZombieOrDead, id)
	
	if (is_user_bot(id))
		bitsum_add(g_bitIsBot, id)
}

public fw_EventPlayerResetHUD(id)
{
	remove_task(id)
	
	set_task(0.1, "TaskResetHUD", id)
}

public TaskResetHUD(const iID)
	if (g_iJetPackEntityID[iID])
		ComputeColor(g_iJetPackEntityID[iID], _, iID)

public zpnm_register_ham_czbots(id)
{
	RegisterHamFromEntity(Ham_Spawn, id, "fw_HamPlayerSpawn_Post", 1)
	RegisterHamFromEntity(Ham_Killed, id, "fw_HamPlayerKilled_Post", 1)
//	RegisterHamFromEntity(Ham_Player_Jump, id, "fw_HamPlayerJump")
	//RegisterHamFromEntity(Ham_TakeDamage, id, "fw_HamPlayerTakeDamage")
}
/*
public fw_HamPlayerTakeDamage(id, inflictor, attacker, Float:damage, damage_type)
{
	// Prevent from taking fall damage when having a jetpack
	if (!g_iJetPackEntityID[id] || !(damage_type & (1<<5)))
		return HAM_IGNORED;
	
	return HAM_SUPERCEDE;
}*/

public client_disconnect(id)
{
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Disconnected.", id)
	#endif
	*/
	bitsum_del(g_bitIsBot, id)
	bitsum_add(g_bitZombieOrDead, id)
	
	remove_task(id)
	
	DropJetPack(id)
}

public fw_HamPlayerSpawn_Post(id)
{
	if (!is_user_alive(id))
		return;
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Ham spawn post.", id)
	#endif
	*/
	remove_task(id)
	
	bitsum_del(g_bitZombieOrDead, id)
	
	if (!g_iJetPackEntityID[id])
		return;
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Has jetpack with entity id %d.", id, g_iJetPackEntityID[id])
	#endif
	*/
	set_pev(g_iJetPackEntityID[id], pev_iuser1, id)
	ProgressBar(id, 0)
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Setting fuel level to 0.", id)
	#endif
	*/
	set_pev(g_iJetPackEntityID[id], g_fFuelConsumed, 0.0)
	
	ComputeColor(g_iJetPackEntityID[id], 0.0, id, false)
}

public zpnm_user_spawn_post(id)
{
	bitsum_del(g_bitZombieOrDead, id)
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] ZPNM Ham spawn post.", id)
	#endif
	*/
	if (bitsum_get(g_bitIsBot, id))
	{
/*		#if defined DefDebug
		log_to_file(g_szFileDebug, "[Player %d] Is a bot. Forcing buy of jetpack.", id)
		#endif
		*/
		zp_force_buy_extra_item(id, g_iItemJetPackID)
	}
}

public zp_extra_item_selected(id, item)
{
	if (item != g_iItemJetPackID)
		return PLUGIN_CONTINUE;
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Jetpack item selected.", id)
	#endif
	*/
	if (g_bEscapeMap)// && !(get_user_flags(id) & ADMIN_RCON)
	{
/*		#if defined DefDebug
		log_to_file(g_szFileDebug, "[Player %d] Escape map running and player doesn't have Rcon access. Canceling item buy.", id)
		#endif
		*/
		colored_chat(id, "^3You can't buy jetpack on escape maps^1!")
		
		return ZP_PLUGIN_HANDLED;
	}
	else if (g_iJetPackEntityID[id])
	{
/*		#if defined DefDebug
		log_to_file(g_szFileDebug, "[Player %d] Already has a jetpack with entity ID %d. Canceling item buy.", id, g_iJetPackEntityID[id])
		#endif
		*/
		colored_chat(id, "^3You already have a jetpack^1!")
		
		return ZP_PLUGIN_HANDLED;
	}
	
	new iEntityID = create_entity("info_target");
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Has bought a jetpack. Entity created with ID %d.", id, iEntityID)
	#endif
	*/
	if (is_valid_ent(iEntityID))
	{
/*		#if defined DefDebug
		log_to_file(g_szFileDebug, "[Player %d] Jetpack entity %d is valid. Adding it to the entity list array.", id, iEntityID)
		#endif
		*/
		remove_task(id)
		
		g_iJetPackEntityID[id] = iEntityID;
		
		ArrayPushCell(g_ArrayJetPackEntityList, iEntityID)
		
		set_pev(iEntityID, g_fFuelConsumed, 0.0)
		engfunc(EngFunc_SetModel, iEntityID, g_szJetPackModelP)
		set_pev(iEntityID, pev_movetype, MOVETYPE_FOLLOW)
		set_pev(iEntityID, pev_aiment, id)
		set_pev(iEntityID, pev_classname, g_szClassNameJetPack)
		set_pev(iEntityID, pev_solid, SOLID_NOT)
		set_pev(iEntityID, pev_iuser1, id)
		set_pev(iEntityID, pev_owner, id)
		set_pev(iEntityID, pev_rendermode, kRenderNormal)
		set_pev(iEntityID, pev_renderfx, kRenderFxGlowShell)
		set_pev(iEntityID, pev_renderamt, random_float(5.77, 7.57))
		
		ComputeColor(iEntityID, 0.0, id, false)
		
		emit_sound(id, CHAN_ITEM, g_szJetPackSoundPickUp, VOL_NORM, ATTN_NORM, 0, PITCH_NORM)
		
		colored_chat(id, "JetPack usage: ^4JUMP^1 + ^4DUCK^1 = ^4Fly^1 | Loading bar = Cooldown BEFORE fuel regen start")
/*		
		#if defined DefDebug
		log_to_file(g_szFileDebug, "[Player %d] Entity %d succesfully created and alocated.", id, iEntityID)
		#endif*/
	}
	
	return PLUGIN_CONTINUE;
}

public zp_user_humanized_post(id)
{
	bitsum_del(g_bitZombieOrDead, id)
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] ZPNM humanized post.", id)
	#endif
	*/
	if (bitsum_get(g_bitIsBot, id))
	{
/*		#if defined DefDebug
		log_to_file(g_szFileDebug, "[Player %d] Is a bot. Forcing buy of jetpack.", id)
		#endif
		*/
		zp_force_buy_extra_item(id, g_iItemJetPackID)
	}
}

public fw_ClCmdDrop(id)
{
	if (bitsum_get(g_bitZombieOrDead, id) || get_user_weapon(id) != CSW_KNIFE)
		return PLUGIN_CONTINUE;
	
	return DropJetPack(id);
}

public zp_user_infected_post(id)
{
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Infected post.", id)
	#endif
	*/
	bitsum_add(g_bitZombieOrDead, id)
	
	DropJetPack(id)
}

public fw_JetPackTouch(iEntityID, id)
{
	if (g_iJetPackEntityID[id] || bitsum_get(g_bitZombieOrDead, id) || task_exists(iEntityID + g_iTaskEntityEnablePickUpID))
		return;
	
	remove_task(id)
	remove_task(iEntityID + g_iTaskEntityRemoveID)
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Touch jetpack entity with ID %d.", id, iEntityID)
	#endif
	*/
	engfunc(EngFunc_SetModel, iEntityID, g_szJetPackModelP)
	set_pev(iEntityID, pev_movetype, MOVETYPE_FOLLOW)
	set_pev(iEntityID, pev_aiment, id)
	set_pev(iEntityID, pev_solid, SOLID_NOT)
	set_pev(iEntityID, pev_owner, id)
	set_pev(iEntityID, pev_iuser1, id)
	set_pev(iEntityID, pev_rendermode, kRenderNormal)
	set_pev(iEntityID, pev_renderfx, kRenderFxGlowShell)
	set_pev(iEntityID, pev_renderamt, random_float(5.77, 7.57))
	
	g_iJetPackEntityID[id] = iEntityID;
	
	ComputeColor(iEntityID, _, id)
	
	emit_sound(id, CHAN_ITEM, g_szJetPackSoundPickUp, VOL_NORM, ATTN_NORM, 0, PITCH_NORM)
	
	colored_chat(id, "JetPack usage: ^4JUMP^1 + ^4DUCK^1 = ^4Fly^1 | Loading bar = Cooldown BEFORE refuel start")
/*	
	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Touch. Entity %d succesfully attached to player.", id, g_iJetPackEntityID[id])
	#endif*/
}

public fw_HamPlayerKilled_Post(id)
{
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Ham killed post.", id)
	#endif
	*/
	bitsum_add(g_bitZombieOrDead, id)
	
	// Only lose the item permanently if the round has started
	if (zp_has_round_started())
		DropJetPack(id)
	else
	{
		remove_task(id)
		
		if (!g_iJetPackEntityID[id])
			return;
		
		ProgressBar(id, 0)
		IconJetPackFuel(id, _, 0)
		
		set_pev(g_iJetPackEntityID[id], pev_iuser1, id + 32)
	}
}

DropJetPack(const id)
{
	if (!g_iJetPackEntityID[id])
		return PLUGIN_CONTINUE;
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Drop jetpack with entity id %d.", id, g_iJetPackEntityID[id])
	#endif
	*/
	remove_task(id)
	ProgressBar(id, 0)
	IconJetPackFuel(id, _, 0)
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Drop. Setting entity iuser1.", id)
	#endif
	*/
	set_pev(g_iJetPackEntityID[id], pev_iuser1, 0)
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Drop. Setting entity model.", id)
	#endif
	*/
	engfunc(EngFunc_SetModel, g_iJetPackEntityID[id], g_szJetPackModelW)
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Drop. Setting entity solid state.", id)
	#endif
	*/
	set_pev(g_iJetPackEntityID[id], pev_solid, SOLID_TRIGGER)
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Drop. Setting entity move type.", id)
	#endif
	*/
	set_pev(g_iJetPackEntityID[id], pev_movetype, MOVETYPE_TOSS)
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Drop. Setting entity aiment.", id)
	#endif
	*/
	set_pev(g_iJetPackEntityID[id], pev_aiment, 0)
//	engfunc(EngFunc_SetSize, g_iJetPackEntityID[id], Float:{ -5.0, -5.0, -5.0 }, Float:{ 5.0, 5.0, 5.0 })
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Drop. Setting entity origin, velocity and angles.", id)
	#endif
	*/
	new Float:fOrigin[3], Float:fVelocity[3];
	
	pev(id, pev_origin, fOrigin)
	set_pev(g_iJetPackEntityID[id], pev_origin, fOrigin)
	
	pev(id, pev_angles, fOrigin)
	fOrigin[0] = 0.0;
	set_pev(g_iJetPackEntityID[id], pev_angles, fOrigin)
	
	velocity_by_aim(id, 250, fOrigin)
	pev(id, pev_velocity, fVelocity)
	
	fVelocity[0] += fOrigin[0];
	fVelocity[1] += fOrigin[1];
	fVelocity[2] += fOrigin[2];
	
	set_pev(g_iJetPackEntityID[id], pev_velocity, fVelocity)
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Drop. Setting entity next think.", id)
	#endif
	*/
	set_task(get_pcvar_float(g_pCvarPickUpCoolDown), "TaskPickUpEnable", g_iJetPackEntityID[id] + g_iTaskEntityEnablePickUpID) // prevent drop/pickup exploit
	set_task(get_pcvar_float(g_pCvarRemoveAfterDrop), "TaskRemove", g_iJetPackEntityID[id] + g_iTaskEntityRemoveID)
	
	g_iJetPackEntityID[id] = 0;
	
	return PLUGIN_HANDLED;
}

public TaskPickUpEnable()
{
	/* Do nothing */
}

public TaskRemove(iEntityID)
{
	iEntityID -= g_iTaskEntityRemoveID;
	
	RemoveJetPack(iEntityID)
}

ComputeColor(const iID, Float:fFuelConsumed = -1.0, const iUserID, bFlash = -1)
{
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Entity %d] Compute color.", iID)
	#endif
	*/
	if (fFuelConsumed == -1.0)
	{
/*		#if defined DefDebug
		log_to_file(g_szFileDebug, "[Entity %d] Compute color. Getting fuel consumption percentage since it's not defined.", iID)
		#endif
		*/
		pev(iID, g_fFuelConsumed, fFuelConsumed)
	}
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Entity %d] Compute color. Calculating color amounts.", iID)
	#endif
	*/
	static Float:fColorAmounts[3];
	
	if (fFuelConsumed != 100.0)
	{
		fColorAmounts[0] = 255 * fFuelConsumed / 100; // red
		fColorAmounts[1] = 255 - fColorAmounts[0]; // green
		fColorAmounts[2] = 0.0;
	}
	else
	{
		fColorAmounts[0] = 157.0;
		fColorAmounts[1] = 157.0;
		fColorAmounts[2] = 157.0;
	}
	
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Entity %d] Compute color. Setting entity rendering.", iID)
	#endif
	*/
	set_pev(iID, pev_rendercolor, fColorAmounts)
	
	if (!iUserID) // || bitsum_get(g_bitZombieOrDead, iUserID)
	{
/*		#if defined DefDebug
		log_to_file(g_szFileDebug, "[Entity %d] Compute color. User %d is invalid/zombie/dead.", iID, iUserID)
		#endif
		*/
		return;
	}
	
	static iColorAmounts[3];
	
	iColorAmounts[0] = floatround(fColorAmounts[0]);
	iColorAmounts[1] = floatround(fColorAmounts[1]);
	iColorAmounts[2] = floatround(fColorAmounts[2]);
	
	if (bFlash == -1)
	{
/*		#if defined DefDebug
		log_to_file(g_szFileDebug, "[Entity %d] Compute color. Icon flash unspecified. Finding if it should flash or not.", iID)
		#endif
		*/
		if (!pev(iID, g_bIsRefueling))
			bFlash = 0;
		else
			bFlash = 1;
	}
	
	IconJetPackFuel(iUserID, iColorAmounts, !bFlash ? 1 : 2)
}

stock IconJetPackFuel(const iID, const iColorAmounts[3] = { 0, 255, 0 }, const iMode = 1)
{
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Icon display. Colors: %d, %d, %d. Display mode: %d.",
	iID, iColorAmounts[0], iColorAmounts[1], iColorAmounts[2], iMode)
	#endif
	*/
	static msgStatusIcon;
	if (!msgStatusIcon)
		msgStatusIcon = get_user_msgid("StatusIcon");
	
	message_begin(MSG_ONE, msgStatusIcon, _, iID)
	write_byte(iMode) // status (0=hide, 1=show, 2=flash)
	write_string("item_longjump") // sprite name
	write_byte(iColorAmounts[0])
	write_byte(iColorAmounts[1])
	write_byte(iColorAmounts[2])
	message_end()
}

stock ProgressBar(const iID, const iTime, const iStartPercent = 0)
{
/*	#if defined DefDebug
	log_to_file(g_szFileDebug, "[Player %d] Progress bar message. Time: %d. Start position percentage: %d.", iID, iTime, iStartPercent)
	#endif
	*/
	static msgBarTime2;
	if (!msgBarTime2)
		msgBarTime2 = get_user_msgid("BarTime2");
	
	message_begin(MSG_ONE, msgBarTime2, _, iID)
	write_short(iTime)
	write_short(iStartPercent)
	message_end()
}

RemoveJetPack(const iEntitySpecifiedID = 0)
{
	new iArraySizeJetPackList = ArraySize(g_ArrayJetPackEntityList);
	
	if (!iArraySizeJetPackList)
		return;
	
	new iEntityID;
	
	for (new iArrayPosition = iArraySizeJetPackList - 1; iArrayPosition >= 0; iArrayPosition--)
	{
		iEntityID = ArrayGetCell(g_ArrayJetPackEntityList, iArrayPosition);
		
		if (iEntitySpecifiedID && iEntityID != iEntitySpecifiedID || pev(iEntityID, pev_iuser1))
			continue;
		
		ArrayDeleteItem(g_ArrayJetPackEntityList, iArrayPosition)
		
		remove_task(iEntityID + g_iTaskEntityEnablePickUpID)
		remove_task(iEntityID + g_iTaskEntityRemoveID)
		
		remove_entity(iEntityID)
	}
}
