/*================================================================================
	
		***************************************************************
		*********** [Zombie Plague Multi HUD Info 1.1.1] **************
		***************************************************************
	
	----------------------
	-*- Licensing Info -*-
	----------------------
	
	Zombie Plague Multi HUD Info
	by schmurgel1983(@msn.com)
	Copyright (C) 2008-2011 Stefan "schmurgel1983" Focke
	Versions 1.1.0 and higher by georgik57(@gmail.com)
	Copyright (C) 2011-2013 D i 5 7 i n c T
	
	This program is free software: you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation, either version 3 of the License, or (at your
	option) any later version.
	
	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
	Public License for more details.
	
	You should have received a copy of the GNU General Public License along
	with this program. If not, see <http://www.gnu.org/licenses/>.
	
	In addition, as a special exception, the author gives permission to
	link the code of this program with the Half-Life Game Engine ("HL
	Engine") and Modified Game Libraries ("MODs") developed by Valve,
	L.L.C ("Valve"). You must obey the GNU General Public License in all
	respects for all of the code used other than the HL Engine and MODs
	from Valve. If you modify this file, you may extend this exception
	to your version of the file, but you are not obligated to do so. If
	you do not wish to do so, delete this exception statement from your
	version.
	
	No warranties of any kind. Use at your own risk.
	
	-------------------
	-*- Description -*-
	-------------------
	
	Shows stats like csstats with zombie/human remaining.
	Gives every 3 seconds info about the HP from first zombie,
	last human, nemesis, survivor, sniper or all of these classes.
	Shows currently played game mode.
	
	--------------------
	-*- Requirements -*-
	--------------------
	
	* Mods: Counter-Strike 1.6 or Condition-Zero
	* AMXX: Version 1.8.0 or later
	* Module: fakemeta, hamsandwich
	
	-----------------
	-*- Changelog -*-
	-----------------
	
* v1.0.0:
   - Initial release Privat (30th Aug 2008)
   - Initial release Alliedmodders (2nd Feb 2011)
	
* v1.1.0 (1st Nov 2011 - Private):
   - Fully rescripted the plug-in (optimized very much)
   - Removed the hamsandwich module and support for CZ bots
   - Added support for Zombie Plague Nightmare (version 1.9f or later)
	
* v1.1.1 (17th Nov 2011 - Private):
   - Renamed plug-in from "Zombie Plague Stats" to "Zombie Plague Multi HUD Info"
		(Thanks to suNx for the inspiration and idea)
   - Fixed human and zombie count error
   - Added support for showing the current game mode and different colors
   - Added multilingual support
   - Optimized the plug-in a bit more and removed useless code
	
=================================================================================*/

#include <amxmodx>

#if AMXX_VERSION_NUM < 180
	#assert AMX Mod X v1.8.0 or later library required!
#endif

#include <zombieplaguenightmare>

/*================================================================================
 [Global Variables]
=================================================================================*/

// Game vars
new g_iCountPlayers
new g_iCountAliveZombies // zombie count
new g_iCountAliveHumans // human count

new const g_mode_lang[][] = 
{
	"ZPNM_MODE_INFECTION",
	"ZPNM_MODE_NEMESIS",
	"ZPNM_MODE_SURVIVOR",
	"ZPNM_MODE_SWARM",
	"ZPNM_MODE_MULTI",
	"ZPNM_MODE_PLAGUE",
	"ZPNM_MODE_SNIPER",
	"ZPNM_MODE_ASSASSIN"
}

new const g_hud_color_R[] =
{
	255,
	255,
	0,
	0,
	255,
	0,
	0,
	57
}

new const g_hud_color_G[] =
{
	255,
	0,
	0,
	255,
	75,
	255,
	255,
	0
}

new const g_hud_color_B[] =
{
	0,
	0,
	255,
	0,
	0,
	255,
	255,
	255
}

/*================================================================================
 [Init]
=================================================================================*/

public plugin_init()
{
	register_plugin("[ZPNM] Multi HUD Info", "1.1.1", "schmurgel1983, D i 5 7 i n c T")
	
	register_event("HLTV", "event_round_start", "a", "1=0", "2=0")
	
	register_dictionary("statsx.txt")
	register_dictionary("zombie_plague.txt")
	register_dictionary("zpnm_multi_hud_info.txt")
}

public client_putinserver(id)
	g_iCountPlayers++

public client_disconnect(id)
{
	g_iCountPlayers--
	
	if (!is_user_alive(id))
		return;
	
	if (zp_get_user_zombie(id))
		g_iCountAliveZombies--
	else
		g_iCountAliveHumans--
}

/*================================================================================
 [Main Forwards]
=================================================================================*/

// Event Round Start
public event_round_start()
{
	g_iCountAliveZombies = 0
	g_iCountAliveHumans = 0
}

public zp_round_started(mode)
{
	new szMode[2];
	szMode[0] = mode - 1;
	
	display_multi_hud_info(szMode)
	
	set_task(0.57, "display_multi_hud_info", _, szMode, 1, "b")
}

public zp_round_ended()
	remove_task()

public zpnm_user_spawn_post(id)
	g_iCountAliveHumans++

public zp_user_infected_pre(id)
{
	if (zp_get_user_zombie(id))
		return;
	
	g_iCountAliveHumans--
	g_iCountAliveZombies++
}

public zp_user_humanized_pre(id)
{
	if (!zp_get_user_zombie(id))
		return;
	
	g_iCountAliveHumans++
	g_iCountAliveZombies--
}

public zpnm_user_killed_post(id)
{
	if (zp_get_user_zombie(id))
		g_iCountAliveZombies--
	else
		g_iCountAliveHumans--
}

/*================================================================================
 [Other Functions]
=================================================================================*/

public display_multi_hud_info(const mode[])
{
	static msgSyncHUD; // message sync object
	
	if (!msgSyncHUD)
		msgSyncHUD = CreateHudSyncObj();
	
	set_hudmessage(g_hud_color_R[mode[0]], g_hud_color_G[mode[0]], g_hud_color_B[mode[0]], -1.0, 0.13157, 0, 0.0, 0.67, 0.0, 0.0, -1)
	ShowSyncHudMsg(0, msgSyncHUD, "%L: %L^n^n[%L: %d][%L: %d]^n[%L: %d][%L: %d]", LANG_PLAYER, "ZPNM_CURRENT_MODE", LANG_PLAYER, g_mode_lang[mode[0]],
	LANG_PLAYER, "PLAYERS", g_iCountPlayers, LANG_PLAYER, "ZPNM_DEAD_ADJECTIVE_PLURAL", g_iCountPlayers - g_iCountAliveHumans - g_iCountAliveZombies,
	LANG_PLAYER, "MENU_INFO2", g_iCountAliveHumans, LANG_PLAYER, "MENU_INFO3", g_iCountAliveZombies)
}
